
#
# ENCODE in Python
#
# EFile representation of ENCODE BED file. 
#
# Wong Limsoon
# 5 November 2020
#


from synciterators import EIterator
from simplebed import OnDiskBedEFile, Bed
from samples import Sample, OnDiskSampleEFile 
from samples import OnDiskAltSampleEFile, TransientSampleEFile
from samplefileops import OnDiskSampleFile


def toEncodeNPBedEFile(bedEFile):

    """ Name meta attributes in bedEFile according to ENCODE narrow peaks.
    """

    bedEFile._oldEiterator = bedEFile.eiterator

    def newEiterator(self = bedEFile):
        labels = { "_6" : "signalValue", "_7" : "pValue",
                   "_8" : "qValue", "_9" : "peak" }
        tmp = self._oldEiterator()
        return EIterator(
                 ( Bed({ labels.get(i, i): dict[i] for i in dict }) 
                     for e in tmp 
                       for dict in [e.dict()]),
                 lambda : tmp.close()
               )

    bedEFile.eiterator = newEiterator
    return bedEFile



def toEncodeNPSample(sample):

    """ Apply ENCODE narrow peaks naming to the BED file of a sample
    """
 
    toEncodeNPBedEFile(sample.bedFile)
    return sample
     


def toEncodeNPSampleEFile(sampleEFile):
  
    """ Apply ENCODE narrow peaks naming to samples in sampleEFile
    """

    sampleEFile._oldEiterator = sampleEFile.eiterator
    def newEiterator(self = sampleEFile):
        tmp = self._oldEiterator()
        return EIterator(
                 ( toEncodeNPSample(s) for s in tmp ),
                 lambda : tmp.close()
               )
    sampleEFile.eiterator = newEiterator
    return sampleEFile


       
def importEncodeNPSampleEFile(dbPath, dbList, importAs):

    """ Copy a Stefano's sample db into Limsoon's format.
    """

    def nameOf(s): 
       import pathlib
       oldname = pathlib.Path(s.sid) 
       return importAs + "-" + oldname.name
    np = toEncodeNPSampleEFile(OnDiskAltSampleEFile(dbPath, dbList))
    ns = [ s.trackSorted().trackSavedAs(nameOf(s)) for s in np ]
    return TransientSampleEFile(ns).serialized(importAs)
 


def importEncodeNPSampleFile(dbPath, dbList, importAs):
    sampleEFile = importEncodeNPSampleEFile(dbPath, dbList, importAs)
    return OnDiskSampleFile(sampleEFile.filename, sampleEFile.protect)




