

#
# Text EFile 
#
# Represent a plain text file as an EIterator of numbered lines. 
# The line numbers are generated automatically.
#
# Wong Limsoon
# 27 January 2021
#

from fileiterators import EFile, OnDiskEFile, TransientEFile
from synciterators import EIterator


class TextLine:

    """ TextLine represents one line in a text file.
    """

    def  __init__(self, lno, line):
        self.lno = lno
        self.line = line



class TextFileCodec:

    """ Serializer and deserializer for text file.
    """

    def serializer(fh, it):
        for e in it:
            print(e.line, file = fh)

    def deserializer(fh):
        n = 0
        def process(l):
            nonlocal n
            n += 1
            e = l.strip()
            return TextLine(n, e) 
        return EIterator((process(x) for x in fh), lambda: fh.close())



class TextEFile(EFile):
    def __init__(self, vec):
        # vec: [{lno, line}]
        EFile.__init__(
            self,
            vec,
            TextFileCodec.serializer,
            TextFileCodec.deserializer,
            ".txt"
        )
      
    def __onDisk__(self, filename,protect = False):
        return OnDiskTextEFile(filename, protect)

    def __transient__(self, vec):
        return TransientTextEFile(vec)



class TransientTextEFile(TextEFile, TransientEFile):
    def __init__(self, vec):
        TransientEFile.__init__(
          self, 
          vec, 
          TextFileCodec.serializer, 
          TextFileCodec.deserializer, 
          ".txt"
        )


class OnDiskTextEFile(TextEFile, OnDiskEFile):
    def __init__(self, filename, protect = True): 
        OnDiskEFile.__init__(
            self,
            filename,
            TextFileCodec.serializer,
            TextFileCodec.deserializer,
            ".txt",
            protect
        )



# 
# Example
#

"""Usage:

from textefiles import TextLine, TextEFile, TransientTextEFile, OnDiskTextEFile

vecA = TransientTextEFile([TextLine(x, str(x*2)) for x in range(0, 100)])

vecA.serialized("vecA")

vecB = OnDiskTextEFile("vecA")

print("\n*** This isvecB ***\n")

for b in vecB:
    print(b.lno, b.line)

"""





